@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2015-2016 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2015-2017 Jakub Wilk <jwilk@jwilk.net>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the “Software”), to deal
@@ -20,29 +20,46 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import print_function
+
+import base64
+import contextlib
 import distutils.version
 import glob
 import os
 import re
-import shutil
+import signal
 import subprocess as ipc
 import sys
-import tempfile
 import time
+import warnings
+
+try:
+    # Python >= 3.3
+    from shlex import quote as shell_quote
+except ImportError:
+    # Python << 3.3
+    from pipes import quote as shell_quote
 
 from .tools import (
     SkipTest,
     assert_true,
+    clean_environ,
+    tempdir,
 )
 
 here = os.path.dirname(__file__)
 
+token = base64.b64encode(os.urandom(8))
+if not isinstance(token, str):
+    token = token.decode('ASCII')
+
 def get_afl_version():
     child = ipc.Popen(['afl-fuzz'], stdout=ipc.PIPE)
     version = child.stdout.readline()
     child.stdout.close()
     child.wait()
-    if str != bytes:
+    if str is not bytes:
         version = version.decode('ASCII')
     version = re.sub(r'\x1b\[[^m]+m', '', version)
     match = re.match(r'^afl-fuzz\s+([0-9.]+)b?\b', version)
@@ -56,7 +73,7 @@ def sleep(n):
 def check_core_pattern():
     with open('/proc/sys/kernel/core_pattern', 'rb') as file:
         pattern = file.read()
-        if str != bytes:
+        if str is not bytes:
             pattern = pattern.decode('ASCII', 'replace')
         pattern = pattern.rstrip('\n')
         if pattern.startswith('|'):
@@ -74,21 +91,17 @@ def _test_fuzz(workdir, target, dumb=False):
     have_crash = False
     have_paths = False
     n_paths = 0
-    def setup_env():
-        os.environ['AFL_SKIP_CPUFREQ'] = '1'
-        os.environ['AFL_I_DONT_CARE_ABOUT_MISSING_CRASHES'] = '1'
-        os.environ['AFL_NO_AFFINITY'] = '1'
     with open('/dev/null', 'wb') as devnull:
         with open(workdir + '/stdout', 'wb') as stdout:
-            cmdline = ['py-afl-fuzz', '-i', input_dir, '-o', output_dir, '--', sys.executable, target]
+            cmdline = ['py-afl-fuzz', '-i', input_dir, '-o', output_dir, '--', sys.executable, target, token]
             if dumb:
                 cmdline[1:1] = ['-n']
-            print(cmdline)
+            print('$ ' + ' '.join(shell_quote(arg) for arg in cmdline))
             afl = ipc.Popen(
                 cmdline,
                 stdout=stdout,
                 stdin=devnull,
-                preexec_fn=setup_env,
+                preexec_fn=clean_environ,
             )
     try:
         timeout = 10
@@ -109,7 +122,7 @@ def _test_fuzz(workdir, target, dumb=False):
         raise
     with open(workdir + '/stdout', 'rb') as file:
         stdout = file.read()
-        if str != bytes:
+        if str is not bytes:
             stdout = stdout.decode('ASCII', 'replace')
         print(stdout)
     if not have_crash and '/proc/sys/kernel/core_pattern' in stdout:
@@ -117,17 +130,39 @@ def _test_fuzz(workdir, target, dumb=False):
     assert_true(have_crash, "target program didn't crash")
     assert_true(have_paths, 'target program produced {n} distinct paths'.format(n=n_paths))
 
+@contextlib.contextmanager
+def stray_process_cleanup():
+    # afl-fuzz doesn't always kill the target process:
+    # https://groups.google.com/d/topic/afl-users/E37s4YDti7o
+    try:
+        yield
+    finally:
+        ps = ipc.Popen(['ps', 'ax'], stdout=ipc.PIPE)
+        strays = []
+        for line in ps.stdout:
+            if not isinstance(line, str):
+                line = line.decode('ASCII', 'replace')
+            if token in line:
+                strays += [line]
+        if strays:
+            warnings.warn('stray process{es} left behind:\n{ps}'.format(
+                es=('' if len(strays) == 1 else 'es'),
+                ps=''.join(strays)
+            ), category=RuntimeWarning)
+            for line in strays:
+                pid = int(line.split()[0])
+                os.kill(pid, signal.SIGKILL)
+        ps.wait()
+
 def test_fuzz(dumb=False):
     def t(target):
-        tmpdir = tempfile.mkdtemp(prefix='python-afl.')
-        try:
-            _test_fuzz(
-                workdir=tmpdir,
-                target=os.path.join(here, target),
-                dumb=dumb,
-            )
-        finally:
-            shutil.rmtree(tmpdir)
+        with stray_process_cleanup():
+            with tempdir() as workdir:
+                _test_fuzz(
+                    workdir=workdir,
+                    target=os.path.join(here, target),
+                    dumb=dumb,
+                )
     yield t, 'target.py'
     yield t, 'target_persistent.py'
 
