@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2015-2017 Jakub Wilk <jwilk@jwilk.net>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the “Software”), to deal
@@ -22,17 +22,17 @@
 
 import os
 
+import afl
+
 from .tools import (
     assert_equal
 )
 
-import afl
-
 here = os.path.dirname(__file__)
 docdir = here + '/../doc'
 
 def uopen(path):
-    if str != bytes:
+    if str is not bytes:
         return open(path, 'rt', encoding='UTF-8')
     else:
         return open(path, 'rt')
