@@ -131,6 +131,8 @@ def clean_environ():
     os.environ['AFL_SKIP_CPUFREQ'] = '1'
     os.environ['AFL_I_DONT_CARE_ABOUT_MISSING_CRASHES'] = '1'
     os.environ['AFL_NO_AFFINITY'] = '1'
+    os.environ['AFL_ALLOW_TMP'] = '1'  # AFL >= 2.48b
+    os.environ['PWD'] = '//' + os.getcwd()  # poor man's AFL_ALLOW_TMP for AFL << 2.48b
 
 def run(cmd, stdin='', xstatus=0):
     child = ipc.Popen(
@@ -201,7 +203,7 @@ def fork_isolation(f):
             elif status == 0 and msg == '':
                 pass
             else:
-                raise RuntimeError('unexpected isolated process status {}'.format(status))
+                raise RuntimeError('unexpected isolated process status {0}'.format(status))
 
     return wrapper
 
